package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import java.util.Arrays;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEmployees();
            findAbsoluteDifferenceWithAvgSalary();
            findStandardDeviationOfSalaries();
            findSalariesNearestToThousands();
            findNumberOfPhoneNumbers();
            findEmployeeWithMoreThanOnePhoneNumber();
            findPhoneByIndex();

        } finally {
            entityManagerFactory.close();
        }
    }

    public static void persistEmployees() {
        Employee employee1 = Employee.create("Diana", 2000, "IT", "111-111-111", "666-666-666", "777-777-777");
        Employee employee2 = Employee.create("Rose", 3500, "Admin", "222-222,222", "888-888-888");
        Employee employee3 = Employee.create("Denise", 2500, "Admin", "333-333-333");
        Employee employee4 = Employee.create("Mike", 4000, "IT", "444-444-444");
        Employee employee5 = Employee.create("Linda", 4500, "Sales", "555-555-555");
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(employee1);
        em.persist(employee2);
        em.persist(employee3);
        em.persist(employee4);
        em.persist(employee5);
        em.getTransaction().commit();

        System.out.println("-- Employees persisted --");
        Query query = em.createQuery(
                "SELECT e FROM Employee e");
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void findAbsoluteDifferenceWithAvgSalary() {
        System.out.println("-- Find salaries' positive difference with average salary --");
        EntityManager em = entityManagerFactory.createEntityManager();
        //find average salary first
        Double avgSal = (Double) em.createQuery("SELECT  AVG(e.salary) from Employee e")
                                   .getSingleResult();
        System.out.println("Average Salary: " + avgSal);
        List<Object[]> list = em.createQuery(
                "SELECT e.name, ABS(e.salary - :avgSal) FROM Employee e")
                                .setParameter("avgSal", avgSal).getResultList();
        list.forEach(o -> System.out.println(Arrays.toString(o)));
        em.close();
    }

    private static void findStandardDeviationOfSalaries() {
        System.out.println("-- Find salaries' standard deviation --");
        EntityManager em = entityManagerFactory.createEntityManager();
        //find average salary
        Double avgSal = (Double) em.createQuery("SELECT AVG(e.salary) from Employee e")
                                   .getSingleResult();
        //find square root of squared sum salary difference from avg
        Double deviationSum = (Double) em
                .createQuery("SELECT SQRT(SUM((e.salary - :avgSal)*(e.salary - :avgSal)))"
                        + " from Employee e").setParameter("avgSal", avgSal).getSingleResult();

        //find square root of employee's count
        Double sampleSizeSquaredRoot = (Double) em.createQuery("SELECT SQRT(COUNT(e)) from Employee e")
                                                  .getSingleResult();
        System.out.println("standard deviation: " + (deviationSum / sampleSizeSquaredRoot));
        em.close();
    }

    private static void findSalariesNearestToThousands() {
        System.out.println("-- Find salaries nearest to 1000 --");
        EntityManager em = entityManagerFactory.createEntityManager();

        List<Object[]> list = em.createQuery("SELECT e.name, e.salary - MOD(e.salary, 1000)"
                + " from Employee e").getResultList();
        list.forEach(o -> System.out.println(Arrays.toString(o)));
        em.close();
    }

    private static void findNumberOfPhoneNumbers() {
        System.out.println("-- Find number of phone numbers --");
        EntityManager em = entityManagerFactory.createEntityManager();

        List<Object[]> list = em.createQuery("SELECT e.id, e.name, SIZE(e.phoneNumbers)"
                + " from Employee e GROUP BY e.id, e.name").getResultList();
        list.forEach(o -> System.out.println(Arrays.toString(o)));
        em.close();
    }

    private static void findEmployeeWithMoreThanOnePhoneNumber() {
        System.out.println("-- Find employees who have more than 1 phone number --");
        EntityManager em = entityManagerFactory.createEntityManager();
        List<Employee> list = em.createQuery("SELECT DISTINCT e from Employee e"
                + " JOIN e.phoneNumbers p WHERE INDEX(p) > 0")
                                .getResultList();
        list.stream().forEach(System.out::println);
    }

    private static void findPhoneByIndex() {
        System.out.println("-- Find phones by index in phoneNumbers list --");
        EntityManager em = entityManagerFactory.createEntityManager();
        List<Object[]> list = em.createQuery("Select e.name, INDEX(p), p from Employee e JOIN e.phoneNumbers p")
                                .getResultList();
        list.forEach(arr -> System.out.println(Arrays.toString(arr)));
        em.close();
    }
}